@@ -1,4 +1,4 @@
-length3 <- function(design, with.blocks=FALSE){
+length3 <- function(design, with.blocks=FALSE, J=FALSE){
     ## function to calculate number of generalized words of length 3
     ## according to Xu and Wu 2001 Annals
     
@@ -57,8 +57,8 @@ length3 <- function(design, with.blocks=FALSE){
     anz <- 0
     for (i in 1:ncol(triples))
        anz <- anz+prod(nlevels[triples[,i,drop=FALSE]]-1)
-    mm3 <- matrix(NA,nrow(design), anz)
-    ## colnames(mm3) <- rep("sp",ncol(mm3))   ## takes too long
+    vec3 <- rep(NA, anz)
+    ## names(vec3) <- rep("sp",length(vec3))   ## takes too long
 
     zaehl <- 1   ## column accessor
 
@@ -71,11 +71,16 @@ length3 <- function(design, with.blocks=FALSE){
           for (a in icols){
            for (b in jcols){
              for (c in kcols){
-              mm3[,zaehl] <- mm[,a]*mm[,b]*mm[,c]
-              ##colnames(mm3)[zaehl] <- paste(i,j,k,sep=":")
+              vec3[zaehl] <- sum(mm[,a]*mm[,b]*mm[,c])
+              ##names(vec3)[zaehl] <- paste(i,j,k,sep=":")
               zaehl <- zaehl+1
            }}}
        }
     }}
-    sum(colSums(mm3)^2)/(n^2)
+    if (J) {
+        names(vec3) <- unlist(apply(triples,2,function(obj) rep(paste((1:nfac)[obj],collapse=":"), prod(nlevels[obj]-1))) )
+        return(abs(vec3)/n)
+        }
+    else 
+    sum(vec3^2)/(n^2)
 }
\ No newline at end of file
