@@ -1,7 +1,7 @@
-length4 <- function(design, with.blocks=FALSE, separate=FALSE){
+length4 <- function(design, with.blocks=FALSE, separate=FALSE, J=FALSE){
     ## function to calculate generalized words of length 4
     ## according to Xu and Wu 2001 Annals
-
+    if (separate & J) stop("separate and J must not both be TRUE")
     ## it might be helpful to locate non-zeros
     ## this is so far not done
     n <- nrow(design)
@@ -62,7 +62,7 @@ length4 <- function(design, with.blocks=FALSE, separate=FALSE){
        }
     else for (i in 1:ncol(quadruples))
        anz <- anz+prod(nlevels[quadruples[,i]]-1)
-    mm4 <- matrix(NA,nrow(design), anz)
+    vec4 <- rep(NA, anz)
   #  if (!separate) rm(quadruples)
     ##namvec <- rep("sp",ncol(mm4))  ## column names, only not attached as such
     ## takes up time, not needed because of quadruples
@@ -82,8 +82,8 @@ length4 <- function(design, with.blocks=FALSE, separate=FALSE){
            for (b in jcols){
              for (c in kcols){
              for (d in lcols){
-              mm4[,zaehl] <- mm[,a]*mm[,b]*mm[,c]*mm[,d]
-              ## namvec[zaehl] <- paste(i,j,k,l,sep=":") ## update column names
+              vec4[zaehl] <- sum(mm[,a]*mm[,b]*mm[,c]*mm[,d])
+              ## namvec[zaehl] <- paste(i,j,k,l,sep=":") ## update names
               zaehl <- zaehl+1
            }}}}
        }
@@ -96,11 +96,16 @@ length4 <- function(design, with.blocks=FALSE, separate=FALSE){
                    ## logical selecting columns from quadruples = mm4
               ## try and expand to complete matrix ?
               cols <- unlist(mapply(":", col.from[sel], col.to[sel]))
-              aus[i] <- sum(colSums(mm4[,cols,drop=FALSE])^2)/(n^2)
+              aus[i] <- sum(vec4[cols]^2)/(n^2)
               }
         names(aus) <- apply(fi2s, 2, "paste", collapse=":")
         return(aus)
     }
+    else{
+    if (J) {
+        names(vec4) <- unlist(apply(quadruples,2,function(obj) rep(paste((1:nfac)[obj],collapse=":"), prod(nlevels[obj]-1))) )
+        return(abs(vec4)/n)
+        }
     else
-    sum(colSums(mm4)^2)/(n^2)
+    sum(vec4^2)/(n^2)}
 }
\ No newline at end of file
